/*
 Copyright (C) JAEHYUK CHO
 All rights reserved.
 Code by JaeHyuk Cho <mailto:minzkn@minzkn.com>
*/

#if !defined(__def_mzapi_source_main_c__)
#define __def_mzapi_source_main_c__ "main.c"

#include "mzdes.h"

#if (!defined(HAVE_CONFIG_H)) || (defined(HAVE_CONFIG_H) && defined(HAVE_STDIO_H))
# include <stdio.h>
#endif
#if (!defined(HAVE_CONFIG_H)) || (defined(HAVE_CONFIG_H) && defined(HAVE_STDLIB_H))
#include <stdlib.h>
#endif
#if (!defined(HAVE_CONFIG_H)) || (defined(HAVE_CONFIG_H) && defined(HAVE_STRING_H))
#include <string.h>
#endif

static void mzapi_tdump(char *s_title, void *s_data, int s_size)
{
 int s_o,s_w,s_i;unsigned char s_b[17];
 (void)fprintf(stdout, "%s:\n", s_title);
 s_b[16]='\0';s_o=(int)0;
 while(s_o<(s_size)){
  s_w=((s_size)-s_o)<16?((s_size)-s_o):16;printf("%08X",s_o);for(s_i=0;s_i<s_w;s_i++){if(s_i==8)printf(" | ");else printf(" ");
  s_b[s_i]=*(((unsigned char *)(s_data))+s_o+s_i);printf("%02X",s_b[s_i]);if((s_b[s_i]&0x80)||(s_b[s_i]<' '))s_b[s_i]='.';}
  while(s_i<16){if(s_i==8)printf("     ");else printf("   ");s_b[s_i]=' ';s_i++;}
  printf(" [%s]\n",(char *)s_b);s_o+=16;}
 (void)fprintf(stdout, "\n");
}

static void test_des(void)
{
 unsigned char s_user_key[ __def_mzapi_des_user_key_size__ ];
 unsigned char s_round_key[ __def_mzapi_des_round_key_size__ ]; /* (16 * 48) bytes */
 char s_data[] = {
  "DES encrypt/decrypt library/example source\n"
  "Copyright (C) JAEHYUK CHO\n"
  "All rights reserved.\n"
  "Code by JaeHyuk Cho <mailto:minzkn@minzkn.com>\n"
 };
 void *s_padding_data;
 int s_data_size, s_padding_data_size;
 
 (void)fprintf(stdout, "\x1b[1;33mtest des\x1b[0m\n~~~~~~~~\n\n");

 /* password */
 (void)memset(&s_user_key[0], 0, sizeof(s_user_key));
 (void)strncpy((char *)(&s_user_key[0]), "ABCDEFGH", sizeof(s_user_key));
 
 mzapi_tdump("user key", &s_user_key[0], sizeof(s_user_key));

 (void)mzapi_des_make_round_key(&s_round_key[0], &s_user_key[0]);
#if 1 /* show round key */
 do
 {
  int s_index0, s_index1, s_index2;
  (void)fprintf(stdout, "round key:\n");
  for(s_index0 = 0;s_index0 < 16;s_index0++)
  {
   (void)fprintf(stdout, "[%2d] ", s_index0);
   for(s_index1 = 0;s_index1 < 48;s_index1 += 8)
   {
    for(s_index2 = 0;s_index2 < 8;s_index2++)
    {
     (void)fprintf(stdout, "%d",
      (unsigned int)s_round_key[s_index0 + s_index1 + s_index2]);
    }
    (void)fprintf(stdout, "b ");
   }
   (void)fprintf(stdout, "\n");
  }
  (void)fprintf(stdout, "\n");
 }while(0);
#endif

 /* padding process */
 s_data_size = sizeof(s_data);
 s_padding_data_size = s_data_size + (__def_mzapi_des_block_size__ - 1);
 s_padding_data_size -= s_padding_data_size % __def_mzapi_des_block_size__;
 s_padding_data = malloc(s_padding_data_size);
 if(s_padding_data == NULL)
 {
  (void)fprintf(stdout, "allocate error !\n");
  return;
 }
 (void)memcpy(s_padding_data, s_data, s_data_size);
 if((s_padding_data_size - s_data_size) > 0)
 { /* zero padding */
  (void)memset(__mzapi_peek_f__(void *, s_padding_data, s_data_size), 0, s_padding_data_size - s_data_size);
 }
 
 mzapi_tdump("data", &s_data[0], s_data_size);
 
 mzapi_tdump("encrypt",
  mzapi_des_encrypt(s_padding_data, s_padding_data_size, &s_round_key[0]),
  (int)s_padding_data_size);
 
 mzapi_tdump("decrypt",
  mzapi_des_decrypt(s_padding_data, s_padding_data_size, &s_round_key[0]),
  (int)s_data_size);

 free((void *)s_padding_data);
}

static void test_triple_des(void)
{
 unsigned char s_user_key[ __def_mzapi_3des_user_key_size__ ];
 unsigned char s_round_key[ __def_mzapi_3des_round_key_size__ ]; /* (16 * 48 * 3) bytes */
 char s_data[] = {
  "DES encrypt/decrypt library/example source\n"
  "Copyright (C) JAEHYUK CHO\n"
  "All rights reserved.\n"
  "Code by JaeHyuk Cho <mailto:minzkn@minzkn.com>\n"
 };
 void *s_padding_data;
 int s_data_size, s_padding_data_size;
 
 (void)fprintf(stdout, "\x1b[1;33mtest triple des\x1b[0m\n~~~~~~~~~~~~~~~~\n\n");

 /* password */
 (void)memset(&s_user_key[0], 0, sizeof(s_user_key));
 (void)strncpy((char *)(&s_user_key[0]), "ABCDEFGHIJKLMNOPQRSTUVWX", sizeof(s_user_key));
 
 mzapi_tdump("user key", &s_user_key[0], sizeof(s_user_key));

 (void)mzapi_3des_make_round_key(&s_round_key[0], &s_user_key[0]);
#if 1 /* show round key */
 do
 {
  int s_index, s_index0, s_index1, s_index2;
  (void)fprintf(stdout, "round key:\n");
  for(s_index = 0;s_index < 3;s_index++)
  {
   for(s_index0 = 0;s_index0 < 16;s_index0++)
   {
    (void)fprintf(stdout, "[%d][%2d] ", s_index, s_index0);
    for(s_index1 = 0;s_index1 < 48;s_index1 += 8)
    {
     for(s_index2 = 0;s_index2 < 8;s_index2++)
     {
      (void)fprintf(stdout, "%d",
       (unsigned int)s_round_key[(s_index * __def_mzapi_des_round_key_size__) + s_index0 + s_index1 + s_index2]);
     }
     (void)fprintf(stdout, "b ");
    }
    (void)fprintf(stdout, "\n");
   }
  }
  (void)fprintf(stdout, "\n");
 }while(0);
#endif

 /* padding process */
 s_data_size = sizeof(s_data);
 s_padding_data_size = s_data_size + (__def_mzapi_3des_block_size__ - 1);
 s_padding_data_size -= s_padding_data_size % __def_mzapi_3des_block_size__;
 s_padding_data = malloc(s_padding_data_size);
 if(s_padding_data == NULL)
 {
  (void)fprintf(stdout, "allocate error !\n");
  return;
 }
 (void)memcpy(s_padding_data, s_data, s_data_size);
 if((s_padding_data_size - s_data_size) > 0)
 { /* zero padding */
  (void)memset(__mzapi_peek_f__(void *, s_padding_data, s_data_size), 0, s_padding_data_size - s_data_size);
 }
 
 mzapi_tdump("data", &s_data[0], s_data_size);

 mzapi_tdump("encrpyt",
  mzapi_3des_encrypt(s_padding_data, s_padding_data_size, &s_round_key[0]),
  (int)s_padding_data_size);
 
 mzapi_tdump("decrypt",
  mzapi_3des_decrypt(s_padding_data, s_padding_data_size, &s_round_key[0]),
  (int)s_data_size);

 free((void *)s_padding_data);
}

int main(void)
{
 test_des();
 test_triple_des();
 return(0);
}

#endif

/* vim: set expandtab: */
/* End of source */
